@@ -4,7 +4,16 @@
 
 
 setClass("BSgenome",
-    contains="GenomeDescription",
+    ## Expected metadata data are:
+    ## - organism
+    ## - common_name
+    ## - genome
+    ## - provider
+    ## - release_date
+    ## - source_url: permanent URL to the place where the 2bit and/or FASTA
+    ##   files used to produce the "single" and "multiple" sequences can be
+    ##   found (and downloaded).
+    contains="Annotated",
     representation(
         ## Name of the BSgenome data package where the BSgenome object is
         ## defined.
@@ -14,15 +23,11 @@ setClass("BSgenome",
         single_sequences="OnDiskNamedSequences",
         multiple_sequences="RdaCollection",
 
-        ## Permanent URL to the place where the FASTA files that were used
-        ## to produce the "single" and "multiple" sequences above can be
-        ## found (and downloaded).
-        source_url="character",
+        ## Original seqinfo.
+        seqinfo="Seqinfo",
 
         ## Named vector representing the translation table from the original
-        ## seqnames (as stored in self@seqinfo@seqnames, the 'seqinfo' slot
-        ## being inherited from the GenomeDescription class) to the user
-        ## seqnames.
+        ## seqnames (as stored in self@seqinfo@seqnames) to the user seqnames.
         user_seqnames="character",
 
         ## For SNPs injection.
@@ -30,13 +35,7 @@ setClass("BSgenome",
 
         ## Used for caching the single and multiple on-disk sequences.
         .seqs_cache="environment",
-        .link_counts="environment",
-
-        ## TODO: Remove the 2 slots below once all the BSgenome data packages
-        ## with masks have been split in 2 packages (naked BSgenome +
-        ## MaskedBSgenome).
-        nmask_per_seq="integer",
-        masks="RdaCollection"
+        .link_counts="environment"
     )
 )
 
@@ -137,13 +136,42 @@ BUILTIN_MASKNAMES <- c("AGAPS", "AMB", "RM", "TRF")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessors.
+### Accessors
 ###
 
-setMethod("length", "BSgenome", function(x) length(names(x)))
-
+### Metadata.
+setMethod("organism", "BSgenome",
+    function(object) metadata(object)$organism
+)
+setMethod("commonName", "BSgenome",
+    function(object) metadata(object)$common_name
+)
+setMethod("provider", "BSgenome", function(x) metadata(x)$provider)
+setMethod("providerVersion", "BSgenome",
+    function(x)
+    {
+        msg <- c("Using providerVersion() on a ", class(x), " object ",
+                 "is deprecated. Please use 'metadata(x)$genome' instead.")
+        .Deprecated(msg=c("  ", wmsg(msg)))
+        metadata(x)$genome
+    }
+)
+setMethod("releaseDate", "BSgenome", function(x) metadata(x)$release_date)
+setMethod("releaseName", "BSgenome",
+    function(x)
+    {
+        msg <- c("Starting with Bioconductor 3.12, BSgenome objects no ",
+                 "longer have a \"release name\". As a consequence of ",
+                 "this change calling releaseName() on a BSgenome object ",
+                 "now always returns NA and is deprecated.")
+        .Deprecated(msg=c("  ", wmsg(msg)))
+        NA_character_
+    }
+)
 setGeneric("sourceUrl", function(x) standardGeneric("sourceUrl"))
-setMethod("sourceUrl", "BSgenome", function(x) x@source_url)
+setMethod("sourceUrl", "BSgenome", function(x) metadata(x)$source_url)
+
+setMethod("length", "BSgenome", function(x) length(names(x)))
 
 setGeneric("mseqnames", function(x) standardGeneric("mseqnames"))
 setMethod("mseqnames", "BSgenome",
@@ -159,13 +187,14 @@ setMethod("mseqnames", "BSgenome",
 setMethod("names", "BSgenome", function(x) c(seqnames(x), mseqnames(x)))
 
 setGeneric("masknames", function(x) standardGeneric("masknames"))
-setMethod("masknames", "BSgenome",
+
+setMethod("masknames", "BSgenome", function(x) NULL)
+
+setMethod("masknames", "MaskedBSgenome",
     function(x)
     {
-        if (x@nmask_per_seq == 0L)
-            return(NULL)
-        ## TODO: Put this kind of checking in a validity method for BSgenome
-        ## objects (that's what validity methods are for).
+        ## TODO: Put this kind of checking in a validity method for
+        ## MaskedBSgenome objects (that's what validity methods are for).
         if (x@nmask_per_seq > length(BUILTIN_MASKNAMES))
             stop("internal anomaly: x@nmask_per_seq > ",
                  length(BUILTIN_MASKNAMES))
@@ -178,62 +207,69 @@ setMethod("masknames", "BSgenome",
 ### seqinfo() accessor and related.
 ###
 
+setMethod("seqnames", "BSgenome", function(x) unname(x@user_seqnames))
+
 setMethod("seqinfo", "BSgenome",
     function(x)
     {
         ans <- x@seqinfo
-        seqlevels(ans) <- x@user_seqnames
+        seqlevels(ans) <- seqnames(x)
         ans
     }
 )
 
-### This is a restricted "seqinfo<-" method for BSgenome objects that
-### only supports replacement of the sequence names, i.e., except for their
-### sequence names, Seqinfo objects 'value' and 'seqinfo(x)' must be identical.
-setReplaceMethod("seqinfo", "BSgenome",
+### We implement a restricted seqinfo() setter for BSgenome object 'x' that
+### supports altering **only** the seqlevels and/or genome of 'seqinfo(x)'.
+### It does NOT allow subsetting 'seqinfo(x)' (by dropping/reordering some
+### of its seqlevels), or altering its seqlengths or circularity flags!
+### In other words, except for their seqnames() and genome(), Seqinfo
+### objects 'new_seqinfo' and 'old_seqinfo' must be identical. This is all
+### we need to make the seqlevelsStyle() setter work on a BSgenome object.
+.check_new2old_and_new_seqinfo <-
+    function(new2old, new_seqinfo, old_seqinfo, context="")
+{
+    if (length(new_seqinfo) != length(old_seqinfo))
+        stop(wmsg("the supplied 'seqinfo' must have the same ",
+                  "length as the current 'seqinfo'", context))
+    if (!(is.null(new2old) || identical(new2old, seq_along(new_seqinfo))))
+        stop(wmsg("'new2old' can only be set to NULL or ",
+                  "'seq_along(seqinfo(x))'", context))
+    seqnames(old_seqinfo) <- seqnames(new_seqinfo)
+    genome(old_seqinfo) <- genome(new_seqinfo)
+    if (!identical(new_seqinfo, old_seqinfo))
+        stop(wmsg("seqlengths() and isCircular() of the supplied 'seqinfo' ",
+                  "must be identical to seqlengths() and isCircular() of ",
+                  "the current 'seqinfo'", context))
+}
+
+.set_BSgenome_seqinfo <-
     function(x, new2old=NULL,
-             pruning.mode=c("error", "coarse", "fine", "tidy"),
-             value)
-    {
-        if (!is(value, "Seqinfo"))
-            stop("the supplied 'seqinfo' must be a Seqinfo object")
-        IN_THIS_CONTEXT <- paste0("when replacing the 'seqinfo' ",
-                                  "of a BSgenome object")
-        pruning.mode <- match.arg(pruning.mode)
-        if (pruning.mode != "error")
-            stop("'pruning.mode' not supported ", IN_THIS_CONTEXT)
-        x_seqinfo <- seqinfo(x)
-        if (is.null(new2old)) {
-            ## Support no-op seqinfo(x) <- seqinfo(x).
-            if (!identical(value, x_seqinfo))
-                stop("'new2old' must be specified ", IN_THIS_CONTEXT)
-            return(x)
-        }
-        if (length(value) != length(x_seqinfo))
-            stop("the supplied 'seqinfo' must have the same length ",
-                 "as the current 'seqinfo' ", IN_THIS_CONTEXT)
-        if (!identical(new2old, seq_along(value)))
-            stop("'new2old' must be NULL or equal to 'seq_along(value)' ",
-                 IN_THIS_CONTEXT)
-        new_seqnames <- seqnames(value)
-        seqnames(x_seqinfo) <- new_seqnames
-        if (!identical(value, x_seqinfo))
-            stop("the supplied and current 'seqinfo' can differ only ",
-                 "in their sequence names ", IN_THIS_CONTEXT)
-        if (any(new_seqnames %in% mseqnames(x)))
-            stop("the supplied 'seqnames' cannot match any of the ",
-                 "multiple sequence names (as returned by 'mseqnames(x)')")
-        x@user_seqnames[] <- new_seqnames  # using [] to preserve the names
-        x
-    }
-)
+                pruning.mode=c("error", "coarse", "fine", "tidy"),
+                value)
+{
+    if (!is(value, "Seqinfo"))
+        stop(wmsg("the supplied 'seqinfo' must be a Seqinfo object"))
+    context <- paste0(" when replacing the 'seqinfo' of a ",
+                      classNameForDisplay(x), " object")
+    pruning.mode <- match.arg(pruning.mode)
+    if (pruning.mode != "error")
+        stop(wmsg("'pruning.mode' is not supported", context))
+    .check_new2old_and_new_seqinfo(new2old, value, seqinfo(x), context)
+    new_seqnames <- seqnames(value)
+    if (any(new_seqnames %in% mseqnames(x)))
+        stop(wmsg("the supplied 'seqnames' cannot match any of the ",
+                  "multiple sequence names (as returned by 'mseqnames(x)')"))
+    x@user_seqnames[] <- new_seqnames  # using [] to preserve the names
+    genome(x@seqinfo) <- unname(genome(value))
+    x
+}
+
+setReplaceMethod("seqinfo", "BSgenome", .set_BSgenome_seqinfo)
 
 setReplaceMethod("seqnames", "BSgenome",
     function(x, value)
     {
-        x_seqinfo <- seqinfo(x)
-        seqnames(x_seqinfo) <- value
-        seqinfo(x, new2old=seq_along(x_seqinfo)) <- x_seqinfo
+        seqnames(seqinfo(x)) <- value
         x
     }
 )
@@ -261,9 +297,8 @@ setMethod("snplocs", "BSgenome",
 ###
 
 ### 'seqnames' only used for sanity check.
-.makeBSgenomeSeqinfo <- function(single_sequences,
-                                 circ_seqs, provider_version,
-                                 seqnames)
+.make_BSgenome_seqinfo <- function(single_sequences, circ_seqs,
+                                   genome, seqnames)
 {
     seqlengths <- seqlengths(single_sequences)
     if (length(seqnames) == 0L) {
@@ -282,73 +317,53 @@ setMethod("snplocs", "BSgenome",
     Seqinfo(seqnames=seqnames,
             seqlengths=seqlengths,
             isCircular=is_circ,
-            genome=provider_version)
+            genome=genome)
 }
 
 ### NOTES:
-### - In BioC 2.14, the 'seqs_pkgname' and 'masks_pkgname' BSgenome slots
-###   were replaced with the 'pkgname' slot but the 2 corresponding
-###   arguments were kept for backward compatibility with existing
-###   BSgenome packages.
+### - In BioC 2.14, the 'seqs_pkgname' BSgenome slot was renamed 'pkgname'
+###   but the corresponding argument was not renamed for backward
+###   compatibility with existing BSgenome packages.
 ### - In BioC 3.1, the 'species' argument was replaced with the 'common_name'
 ###   argument but the former was kept for backward compatibility with
 ###   existing BSgenome packages.
-BSgenome <- function(organism, common_name, provider, provider_version,
+### - In BioC 3.12, the 'provider_version' argument was replaced with the
+###   'genome' argument, and the 'release_name' became ignored and was only
+###   kept for backward compatibility.
+BSgenome <- function(organism, common_name, genome,
+                     provider, provider_version,
                      release_date, release_name, source_url,
                      seqnames, circ_seqs=NA, mseqnames,
                      seqs_pkgname, seqs_dirpath,
-                     nmask_per_seq=0, masks_pkgname=NA, masks_dirpath=NA,
                      species=NA_character_)
 {
-    twobit_filename <- "single_sequences.2bit"
-    twobit_filepath <- file.path(seqs_dirpath, twobit_filename)
-    if (file.exists(twobit_filepath)) {
-        single_sequences <- TwobitNamedSequences(twobit_filepath)
-    } else {
-        fa_filename <- "single_sequences.fa"
-        fa_filepath <- file.path(seqs_dirpath, fa_filename)
-        farz_filename <- paste0(fa_filename, ".rz")
-        farz_filepath <- file.path(seqs_dirpath, farz_filename)
-        if (file.exists(farz_filepath)) {
-            single_sequences <- FastaNamedSequences(farz_filepath)
-        } else if (file.exists(fa_filepath)) {
-            single_sequences <- FastaNamedSequences(fa_filepath)
-        } else {
-            single_sequences <- RdaNamedSequences(seqs_dirpath, seqnames)
-        }
-    }
-    seqinfo <- .makeBSgenomeSeqinfo(single_sequences,
-                                    circ_seqs, provider_version,
-                                    seqnames)
+    single_sequences <- OnDiskNamedSequences(seqs_dirpath, seqnames=seqnames)
+    if (missing(genome))
+        genome <- provider_version
+    seqinfo <- .make_BSgenome_seqinfo(single_sequences, circ_seqs,
+                                      genome, seqnames)
     seqnames <- seqnames(seqinfo)
     if (missing(common_name))
         common_name <- species
-    genome_description <- GenomeDescription(organism, common_name,
-                                            provider, provider_version,
-                                            release_date, release_name,
-                                            seqinfo)
+    metadata <- list(organism=organism,
+                     common_name=common_name,
+                     genome=genome,
+                     provider=provider,
+                     release_date=release_date,
+                     source_url=source_url)
     if (is.null(mseqnames))
         mseqnames <- character(0)
     multiple_sequences <- RdaCollection(seqs_dirpath, mseqnames)
     names(user_seqnames) <- user_seqnames <- seqnames
-    nmask_per_seq <- as.integer(nmask_per_seq)
-    masks_dirpath <- as.character(masks_dirpath)
-    if (nmask_per_seq == 0L || length(seqnames) == 0L) {
-        masks <- RdaCollection(masks_dirpath, character(0))
-    } else {
-        masks <- RdaCollection(masks_dirpath, paste0(seqnames, ".masks"))
-    }
 
-    new("BSgenome", genome_description,
-        pkgname=seqs_pkgname,
-        single_sequences=single_sequences,
-        multiple_sequences=multiple_sequences,
-        source_url=source_url,
-        user_seqnames=user_seqnames,
-        .seqs_cache=new.env(parent=emptyenv()),
-        .link_counts=new.env(parent=emptyenv()),
-        nmask_per_seq=nmask_per_seq,
-        masks=masks
+    new("BSgenome", metadata=metadata,
+                    pkgname=seqs_pkgname,
+                    single_sequences=single_sequences,
+                    multiple_sequences=multiple_sequences,
+                    seqinfo=seqinfo,
+                    user_seqnames=user_seqnames,
+                    .seqs_cache=new.env(parent=emptyenv()),
+                    .link_counts=new.env(parent=emptyenv())
     )
 }
 
@@ -376,11 +391,46 @@ setMethod("as.list", "BSgenome",
                function(seqname) x[[seqname]])
 )
 
+setAs("BSgenome", "GenomeDescription",
+    function(from)
+    {
+        metadata <- metadata(from)
+        GenomeDescription(organism=metadata$organism,
+                          common_name=metadata$common_name,
+                          provider=metadata$provider,
+                          provider_version=metadata$genome,
+                          release_date=metadata$release_date,
+                          release_name=NA_character_,
+                          seqinfo=seqinfo(from))
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### bsgenomeName()
+###
+
+setMethod("bsgenomeName", "BSgenome",
+    function(x) bsgenomeName(as(x, "GenomeDescription"))
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The "show" method
 ###
 
+.print_BSgenome_metadata <- function(metadata, prefix="")
+{
+    common_name <- metadata$common_name
+    if (!is.na(common_name))
+        cat(common_name, "genome:\n")
+    cat(prefix, "organism: ", metadata$organism,
+                " (", common_name, ")\n", sep="")
+    cat(prefix, "genome: ", metadata$genome, "\n", sep="")
+    cat(prefix, "provider: ", metadata$provider, "\n", sep="")
+    cat(prefix, "release date: ", metadata$release_date, "\n", sep="")
+}
+
 .SHOW_BSGENOME_PREFIX <- "# "
 .SHOW_SEQSECTION_PREFIX <- "#   "
 
@@ -390,10 +440,7 @@ setMethod("show", "BSgenome",
         mystrwrap <- function(line)
             writeLines(strwrap(line, width=getOption("width")+1,
                                exdent=0L, prefix=.SHOW_BSGENOME_PREFIX))
-        if (!is.na(object@common_name))
-            cat(object@common_name, "genome:\n")
-        GenomeInfoDb:::showGenomeDescription(object,
-                                             margin=.SHOW_BSGENOME_PREFIX)
+        .print_BSgenome_metadata(metadata(object), prefix=.SHOW_BSGENOME_PREFIX)
         if (!is.null(SNPlocs_pkgname(object)))
             cat(.SHOW_BSGENOME_PREFIX, "with SNPs injected from package: ", SNPlocs_pkgname(object), "\n", sep="")
         if (length(mseqnames(object)) == 0L) {
@@ -466,7 +513,7 @@ setMethod("show", "BSgenome",
         return(ans)
     }
     # single sequence
-    ans <- x@single_sequences[[seqname]]
+    ans <- getListElement(x@single_sequences, seqname)
     ## Check the length of the sequence
     if (length(ans) != seqlengths(x)[[user_seqname]]) {
         stop(user_seqname, " sequence does not have the expected length. ",
@@ -482,17 +529,17 @@ setMethod("show", "BSgenome",
         masks_objname <- paste0(seqname, ".masks")
         builtinmasks <- x@masks[[masks_objname]]
         if (length(builtinmasks) < nmask_per_seq) {
-            masks_filepath <- rdaPath(x@masks, masks_objname)
+            masks_path <- rdaPath(x@masks, masks_objname)
             stop("expecting ", nmask_per_seq, " built-in masks per ",
                  "single sequence, found only ", length(builtinmasks),
-                 " in file '", masks_filepath, "'. ",
+                 " in file '", masks_path, "'. ",
                  "May be the data on disk is corrupted?")
         }
         if (length(builtinmasks) > nmask_per_seq)
             builtinmasks <- builtinmasks[seq_len(nmask_per_seq)]
         if (!identical(names(builtinmasks), masknames(x))) {
-            masks_filepath <- rdaPath(x@masks, masks_objname)
-            stop("mask names found in file '", masks_filepath, "' are not ",
+            masks_path <- rdaPath(x@masks, masks_objname)
+            stop("mask names found in file '", masks_path, "' are not ",
                  "identical to the names returned by masknames(). ",
                  "May be the data on disk is corrupted?")
         }
@@ -555,10 +602,10 @@ setMethod("[[", "BSgenome",
         ## to be of length <= 1
         if (length(subscripts) == 0L)
             stop("no index specified")
-        i <- subscripts[[1]]
-        if (length(i) < 1)
+        i <- subscripts[[1L]]
+        if (length(i) < 1L)
             stop("attempt to select less than one element")
-        if (length(i) > 1)
+        if (length(i) > 1L)
             stop("attempt to select more than one element")
         if (is.character(i)) {
             user_seqname <- try(match.arg(i, names(x)), silent=TRUE)
