@@ -10,6 +10,7 @@ library(BSgenome)
 BSgenome_datapkgs <- c(
   "BSgenome.Amellifera.BeeBase.assembly4",
   "BSgenome.Amellifera.UCSC.apiMel2",
+  "BSgenome.Amellifera.NCBI.AmelHAv3.1",
   "BSgenome.Athaliana.TAIR.04232008",
   "BSgenome.Athaliana.TAIR.TAIR9",
   "BSgenome.Athaliana.TAIR.TAIR10.1",
@@ -25,6 +26,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Cfamiliaris.UCSC.canFam2",
   "BSgenome.Cfamiliaris.UCSC.canFam3",
   "BSgenome.Cjacchus.UCSC.calJac3",
+  "BSgenome.Creinhardtii.JGI.v5.6",
   "BSgenome.Dmelanogaster.UCSC.dm2",
   "BSgenome.Dmelanogaster.UCSC.dm3",
   "BSgenome.Dmelanogaster.UCSC.dm6",
@@ -56,6 +58,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Mmusculus.UCSC.mm8",
   "BSgenome.Mmusculus.UCSC.mm9",
   "BSgenome.Mmusculus.UCSC.mm10",
+  "BSgenome.Mmusculus.UCSC.mm39",
   "BSgenome.Ppaniscus.UCSC.panPan1",
   "BSgenome.Ppaniscus.UCSC.panPan2",
   "BSgenome.Ptroglodytes.UCSC.panTro2",
@@ -65,6 +68,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Rnorvegicus.UCSC.rn4",
   "BSgenome.Rnorvegicus.UCSC.rn5",
   "BSgenome.Rnorvegicus.UCSC.rn6",
+  "BSgenome.Rnorvegicus.UCSC.rn7",
   "BSgenome.Scerevisiae.UCSC.sacCer1",
   "BSgenome.Scerevisiae.UCSC.sacCer2",
   "BSgenome.Scerevisiae.UCSC.sacCer3",
