@@ -21,10 +21,9 @@
     bsgenome <- BSgenome(
         organism="@ORGANISM@",
         common_name="@COMMONNAME@",
+        genome="@GENOME@",
         provider="@PROVIDER@",
-        provider_version="@PROVIDERVERSION@",
         release_date="@RELEASEDATE@",
-        release_name="@RELEASENAME@",
         source_url="@SOURCEURL@",
         seqnames=.seqnames,
         circ_seqs=.circ_seqs,
